@@ -1,14 +1,14 @@
-from typing import Dict, Optional, Union, Tuple
 import base64
 import logging
 import cv2
 import threading
 import time
-import tempfile
-import os
 import hashlib
+import numpy as np
 from datetime import datetime, timezone
-from matrice.deploy.utils.kafka_utils import MatriceKafkaDeployment
+from typing import Dict, Optional, Union, Tuple, Any
+from matrice.deploy.stream.kafka_stream import MatriceKafkaDeployment
+from matrice.deploy.optimize.transmission import ClientTransmissionHandler
 
 
 class ClientStreamUtils:
@@ -18,6 +18,9 @@ class ClientStreamUtils:
         service_id: str,
         consumer_group_id: str = None,
         consumer_group_instance_id: str = None,
+        threshold_a: float = 0.95,
+        threshold_b: float = 0.85,
+        enable_intelligent_transmission: bool = True,
     ):
         """Initialize ClientStreamUtils.
 
@@ -26,6 +29,9 @@ class ClientStreamUtils:
             service_id: ID of the deployment
             consumer_group_id: Kafka consumer group ID
             consumer_group_instance_id: Unique consumer group instance ID to prevent rebalancing
+            threshold_a: High similarity threshold for skipping transmission (default: 0.95)
+            threshold_b: Medium similarity threshold for difference transmission (default: 0.85)
+            enable_intelligent_transmission: Whether to enable intelligent frame transmission
         """
         self.streaming_threads = []
         self.session = session
@@ -42,6 +48,28 @@ class ClientStreamUtils:
         self._stop_streaming = False
         self.video_start_times = {}  # Track video start times for timestamp calculation
 
+        # Intelligent transmission components
+        self.enable_intelligent_transmission = enable_intelligent_transmission
+        self.threshold_a = threshold_a
+        self.threshold_b = threshold_b
+        self.txh = ClientTransmissionHandler(
+            threshold_a=threshold_a, threshold_b=threshold_b
+        )
+
+        # Transmission statistics
+        self.frames_sent = 0
+        self.frames_skipped = 0
+        self.frames_diff_sent = 0
+        self.bytes_saved = 0
+
+    def _determine_transmission_strategy(
+        self, frame: np.ndarray, stream_key: str
+    ) -> Tuple[str, Dict]:
+        if not self.enable_intelligent_transmission:
+            self.txh.frame_cache[stream_key] = frame.copy()
+            return "full", {"reason": "intelligent_disabled"}
+        return self.txh.decide_transmission(frame, stream_key)
+
     def _validate_stream_params(
         self, fps: int, quality: int, width: Optional[int], height: Optional[int]
     ) -> bool:
@@ -176,7 +204,9 @@ class ClientStreamUtils:
             else:
                 # For video files, check if we should restart or stop
                 if simulate_video_file_stream:
-                    logging.info(f"End of video file reached, restarting from beginning: {input}")
+                    logging.info(
+                        f"End of video file reached, restarting from beginning: {input}"
+                    )
                     cap.release()
                     time.sleep(10)  # Brief pause before reopening
                     cap = cv2.VideoCapture(input)
@@ -211,7 +241,7 @@ class ClientStreamUtils:
             self.input_order[key] = 0
         self.input_order[key] += 1
         return self.input_order[key]
-    
+
     def _get_video_format(self, input: Union[str, int]) -> str:
         """Get video format extension from input."""
         if isinstance(input, str) and "." in input:
@@ -222,96 +252,30 @@ class ClientStreamUtils:
         """Calculate how many frames to skip for RTSP streams to achieve target FPS."""
         if original_fps <= 0 or target_fps <= 0:
             return 1
-        
+
         frame_skip = max(1, int(original_fps / target_fps))
-        logging.info(f"Original FPS: {original_fps}, Target FPS: {target_fps}, Frame skip: {frame_skip}")
+        logging.info(
+            f"Original FPS: {original_fps}, Target FPS: {target_fps}, Frame skip: {frame_skip}"
+        )
         return frame_skip
 
-    def _build_stream_metadata(
-        self,
-        input: Union[str, int],
-        stream_key: Optional[str],
-        video_props: Dict,
-        fps: int,
-        quality: int,
-        actual_width: int,
-        actual_height: int,
-        stream_type: str,
-        frame_counter: int,
-        is_video_chunk: bool = False,
-        chunk_duration_seconds: Optional[float] = None,
-        chunk_frames: Optional[int] = None,
-    ) -> Dict:
-        """Build consistent metadata for both frame and video chunk streams."""
-        original_fps = video_props["original_fps"]
-        frame_sample_rate = original_fps / fps if original_fps > 0 else 1.0
-        
-        # Calculate chunk duration based on whether it's a video chunk or frame
-        if is_video_chunk and chunk_duration_seconds is not None:
-            duration = chunk_duration_seconds
-            frame_count = chunk_frames if chunk_frames is not None else int(duration * fps)
-        else:
-            duration = 1.0 / fps
-            frame_count = 1
-
-        metadata = {
-            "fps": fps,
-            "original_fps": original_fps,
-            "frame_sample_rate": frame_sample_rate,
-            "video_timestamp": self._calculate_video_timestamp(
-                stream_key, frame_counter, original_fps
-            ),
-            "start_frame": frame_counter,
-            "end_frame": frame_counter + frame_count - 1,
-            "quality": quality,
-            "width": actual_width,
-            "height": actual_height,
-            "is_video_chunk": is_video_chunk,
-            "chunk_duration_seconds": duration,
-            "video_properties": video_props,
-            "video_format": self._get_video_format(input),
-            "stream_type": stream_type,
-        }
-        return metadata
-
-    def _validate_stream_params(
-        self, fps: int, quality: int, width: Optional[int], height: Optional[int]
-    ) -> bool:
-        """Validate common streaming parameters."""
-        if fps <= 0:
-            logging.error("FPS must be positive")
-            return False
-        if quality < 1 or quality > 100:
-            logging.error("Quality must be between 1 and 100")
-            return False
-        if width is not None and width <= 0:
-            logging.error("Width must be positive")
-            return False
-        if height is not None and height <= 0:
-            logging.error("Height must be positive")
-            return False
-        return True
-
-    def _check_stream_support(self) -> bool:
-        """Check if streaming is supported."""
-        if not self.stream_support:
-            logging.error(
-                "Kafka stream support not available, Please check if Kafka is enabled in the deployment and reinitialize the client"
-            )
-            return False
-        return True
+    # Removed: metadata building is centralized in ClientTransmissionHandler
 
     def start_stream(
         self,
         input: Union[str, int],
         fps: int = 10,
         stream_key: Optional[str] = None,
+        stream_group_key: Optional[str] = None,
         quality: int = 95,
         width: Optional[int] = None,
         height: Optional[int] = None,
         simulate_video_file_stream: bool = False,
+        is_video_chunk: bool = False,
+        chunk_duration_seconds: Optional[float] = None,
+        chunk_frames: Optional[int] = None,
     ) -> bool:
-        """Start a stream input to the Kafka stream."""
+        """Start a stream input to the Kafka stream in the current thread."""
         if not self._check_stream_support():
             return False
 
@@ -319,10 +283,22 @@ class ClientStreamUtils:
             return False
 
         try:
-            self._stream_inputs(input, fps, stream_key, quality, width, height, simulate_video_file_stream)
+            self._stream_inputs(
+                input=input,
+                fps=fps,
+                stream_key=stream_key,
+                stream_group_key=stream_group_key,
+                quality=quality,
+                width=width,
+                height=height,
+                simulate_video_file_stream=simulate_video_file_stream,
+                is_video_chunk=is_video_chunk,
+                chunk_duration_seconds=chunk_duration_seconds,
+                chunk_frames=chunk_frames,
+            )
             return True
         except Exception as exc:
-            logging.error("Failed to start streaming thread: %s", str(exc))
+            logging.error("Failed to start streaming: %s", str(exc))
             return False
         except KeyboardInterrupt:
             logging.info("Keyboard interrupt, stopping streaming")
@@ -339,8 +315,11 @@ class ClientStreamUtils:
         width: Optional[int] = None,
         height: Optional[int] = None,
         simulate_video_file_stream: bool = False,
+        is_video_chunk: bool = False,
+        chunk_duration_seconds: Optional[float] = None,
+        chunk_frames: Optional[int] = None,
     ) -> bool:
-        """Add a stream input to the Kafka stream."""
+        """Start a stream input to the Kafka stream in a background thread."""
         if not self._check_stream_support():
             return False
 
@@ -350,7 +329,19 @@ class ClientStreamUtils:
         try:
             thread = threading.Thread(
                 target=self._stream_inputs,
-                args=(input, fps, stream_key, stream_group_key, quality, width, height, simulate_video_file_stream),
+                args=(
+                    input,
+                    fps,
+                    stream_key,
+                    stream_group_key,
+                    quality,
+                    width,
+                    height,
+                    simulate_video_file_stream,
+                    is_video_chunk,
+                    chunk_duration_seconds,
+                    chunk_frames,
+                ),
                 daemon=True,
             )
             self.streaming_threads.append(thread)
@@ -370,6 +361,9 @@ class ClientStreamUtils:
         width: Optional[int] = None,
         height: Optional[int] = None,
         simulate_video_file_stream: bool = False,
+        is_video_chunk: bool = False,
+        chunk_duration_seconds: Optional[float] = None,
+        chunk_frames: Optional[int] = None,
     ) -> None:
         """Stream inputs from a video source to Kafka."""
         quality = max(1, min(100, quality))
@@ -392,7 +386,9 @@ class ClientStreamUtils:
             frame_skip = 1
             is_rtsp_stream = isinstance(input, str) and input.startswith("rtsp")
             if is_rtsp_stream and video_props["original_fps"] > fps:
-                frame_skip = self._calculate_frame_skip(video_props["original_fps"], fps)
+                frame_skip = self._calculate_frame_skip(
+                    video_props["original_fps"], fps
+                )
 
             retry_count = 0
             max_retries = 3
@@ -418,7 +414,13 @@ class ClientStreamUtils:
 
                     try:
                         cap, retry_count = self._handle_frame_read_failure(
-                            input, cap, retry_count, max_retries, width, height, simulate_video_file_stream
+                            input,
+                            cap,
+                            retry_count,
+                            max_retries,
+                            width,
+                            height,
+                            simulate_video_file_stream,
                         )
                     except (RuntimeError, StopIteration):
                         break
@@ -441,43 +443,46 @@ class ClientStreamUtils:
                 # Resize frame if needed
                 frame = self._resize_frame_if_needed(frame, width, height)
 
-                # Encode frame
+                # Prepare payload and metadata via transmission handler
                 try:
-                    encode_params = [cv2.IMWRITE_JPEG_QUALITY, quality]
-                    _, buffer = cv2.imencode(".jpg", frame, encode_params)
-                except Exception as encode_exc:
-                    logging.warning(
-                        f"Failed to encode frame with quality {quality}, using default: {encode_exc}"
-                    )
-                    try:
-                        _, buffer = cv2.imencode(".jpg", frame)
-                    except Exception as fallback_exc:
-                        logging.error(
-                            f"Failed to encode frame even with default settings: {fallback_exc}"
+                    input_bytes, frame_metadata, strategy = (
+                        self.txh.prepare_transmission(
+                            frame,
+                            stream_key=stream_key or "default",
+                            input_source=input,
+                            video_props=video_props,
+                            fps=fps,
+                            quality=quality,
+                            actual_width=actual_width,
+                            actual_height=actual_height,
+                            stream_type=stream_type,
+                            frame_counter=processed_frame_counter,
+                            is_video_chunk=is_video_chunk,
+                            chunk_duration_seconds=chunk_duration_seconds,
+                            chunk_frames=chunk_frames,
                         )
-                        continue
+                    )
+                except Exception as exc:
+                    logging.error(f"Transmission preparation failed: {exc}")
+                    continue
 
-                # Build metadata using unified method
-                frame_metadata = self._build_stream_metadata(
-                    input=input,
-                    stream_key=stream_key,
-                    video_props=video_props,
-                    fps=fps,
-                    quality=quality,
-                    actual_width=actual_width,
-                    actual_height=actual_height,
-                    stream_type=stream_type,
-                    frame_counter=processed_frame_counter,
-                    is_video_chunk=False,
-                )
+                # Update simple counters
+                if strategy == "skip":
+                    self.frames_skipped += 1
+                elif strategy == "difference":
+                    self.frames_diff_sent += 1
 
-                if not self.produce_request(
-                    buffer.tobytes(),
+                # Produce
+                if self.produce_request(
+                    input_bytes,
                     stream_key,
                     stream_group_key,
                     metadata=frame_metadata,
                 ):
-                    logging.warning("Failed to produce frame to Kafka stream")
+                    if strategy == "full":
+                        self.frames_sent += 1
+                else:
+                    logging.warning("Failed to produce to Kafka stream")
 
                 # For non-RTSP streams, maintain desired FPS with time delays
                 if not is_rtsp_stream:
@@ -504,8 +509,8 @@ class ClientStreamUtils:
         stream_group_key: Optional[str] = None,
     ) -> Dict:
         """Construct the input stream dictionary."""
-        if not input_data:
-            logging.error("Input data cannot be empty")
+        if not input_data and (metadata or {}).get("transmission_strategy") != "skip":
+            logging.error("Input data cannot be empty (except for skip strategy)")
             return {}
 
         stream_info = {
@@ -517,13 +522,22 @@ class ClientStreamUtils:
         input_stream = {
             "ip_key_name": self.service_id,
             "stream_info": stream_info,
-            "feed_type": "disk" if metadata.get("stream_type") == "video_file" else "camera",
+            "feed_type": (
+                "disk" if metadata.get("stream_type") == "video_file" else "camera"
+            ),
             "original_fps": metadata.get("original_fps", metadata.get("fps", 30.0)),
             "stream_fps": metadata.get("fps", 30.0),
-            "stream_unit": "segment" if metadata.get("is_video_chunk", False) else "frame",
+            "stream_unit": (
+                "segment" if metadata.get("is_video_chunk", False) else "frame"
+            ),
             "input_order": self._get_next_input_order(stream_key),
-            "frame_count": 1 if not metadata.get("is_video_chunk", False) else int(
-                metadata.get("chunk_duration_seconds", 1.0) * metadata.get("fps", 30)
+            "frame_count": (
+                1
+                if not metadata.get("is_video_chunk", False)
+                else int(
+                    metadata.get("chunk_duration_seconds", 1.0)
+                    * metadata.get("fps", 30)
+                )
             ),
             "start_frame": metadata.get("start_frame"),
             "end_frame": metadata.get("end_frame"),
@@ -547,9 +561,34 @@ class ClientStreamUtils:
                 "last_write_time_sec": "TODO",
                 "last_process_time_sec": "TODO",
             },
-            "content": base64.b64encode(input_data).decode("utf-8"),
-            "input_hash": hashlib.md5(input_data, usedforsecurity=False).hexdigest(),
+            "content": (
+                base64.b64encode(input_data).decode("utf-8") if input_data else ""
+            ),
+            "input_hash": (
+                hashlib.md5(input_data, usedforsecurity=False).hexdigest()
+                if input_data
+                else None
+            ),
         }
+        # Pass through selected metadata fields to server for intelligent transmission/caching
+        passthrough_keys = {
+            "transmission_strategy",
+            "similarity_score",
+            "skip_reason",
+            "difference_metadata",
+            "video_timestamp",
+            "video_properties",
+            "frame_sample_rate",
+            "is_video_chunk",
+            "chunk_duration_seconds",
+            "video_format",
+            "stream_type",
+            "reference_input_hash",
+        }
+        for k, v in (metadata or {}).items():
+            if k in passthrough_keys and v is not None:
+                input_stream[k] = v
+
         return {
             "input_name": f"{input_stream['stream_unit']}_{input_stream['input_order']}",
             "input_unit": input_stream["stream_unit"],
@@ -566,8 +605,12 @@ class ClientStreamUtils:
     ) -> bool:
         """Simple function to produce a stream request to Kafka."""
         try:
-            message = self._construct_input_stream(input_data, metadata or {}, stream_key, stream_group_key)
-            self.kafka_deployment.produce_message(message, timeout=timeout, key=stream_key)
+            message = self._construct_input_stream(
+                input_data, metadata or {}, stream_key, stream_group_key
+            )
+            self.kafka_deployment.produce_message(
+                message, timeout=timeout, key=stream_key
+            )
             return True
         except Exception as exc:
             logging.error("Failed to produce request: %s", str(exc))
@@ -591,11 +634,11 @@ class ClientStreamUtils:
     ) -> bool:
         """Produce a unified stream request to Kafka asynchronously."""
         try:
-            message = self._construct_input_stream(input_data, metadata or {}, stream_key, stream_group_key)
+            message = self._construct_input_stream(
+                input_data, metadata or {}, stream_key, stream_group_key
+            )
             await self.kafka_deployment.async_produce_message(
-                message=message,
-                timeout=timeout,
-                key=stream_key
+                message=message, timeout=timeout, key=stream_key
             )
             return True
         except Exception as exc:
@@ -614,345 +657,6 @@ class ClientStreamUtils:
             )
             return None
 
-    def start_video_stream(
-        self,
-        input: Union[str, int],
-        fps: int = 10,
-        stream_key: Optional[str] = None,
-        stream_group_key: Optional[str] = None,
-        quality: int = 95,
-        width: Optional[int] = None,
-        height: Optional[int] = None,
-        video_duration: Optional[float] = None,
-        max_frames: Optional[int] = None,
-        video_format: str = "mp4",
-        simulate_video_file_stream: bool = False,
-    ) -> bool:
-        """Start a video stream sending video chunks instead of individual frames."""
-        if not self._check_stream_support():
-            return False
-
-        if not self._validate_stream_params(fps, quality, width, height):
-            return False
-
-        # Additional validation for video-specific parameters
-        if video_duration is not None and video_duration <= 0:
-            logging.error("Video duration must be positive")
-            return False
-        if max_frames is not None and max_frames <= 0:
-            logging.error("Max frames must be positive")
-            return False
-        if video_format not in ["mp4", "avi", "webm"]:
-            logging.error("Video format must be one of: mp4, avi, webm")
-            return False
-
-        try:
-            self._stream_video_chunks(
-                input,
-                fps,
-                stream_key,
-                stream_group_key,
-                quality,
-                width,
-                height,
-                video_duration,
-                max_frames,
-                video_format,
-                simulate_video_file_stream,
-            )
-            return True
-        except Exception as exc:
-            logging.error("Failed to start video streaming thread: %s", str(exc))
-            return False
-        except KeyboardInterrupt:
-            logging.info("Keyboard interrupt, stopping video streaming")
-            self.stop_streaming()
-            return False
-
-    def start_background_video_stream(
-        self,
-        input: Union[str, int],
-        fps: int = 10,
-        stream_key: Optional[str] = None,
-        stream_group_key: Optional[str] = None,
-        quality: int = 95,
-        width: Optional[int] = None,
-        height: Optional[int] = None,
-        video_duration: Optional[float] = None,
-        max_frames: Optional[int] = None,
-        video_format: str = "mp4",
-        simulate_video_file_stream: bool = False,
-    ) -> bool:
-        """Start a background video stream sending video chunks instead of individual frames."""
-        if not self._check_stream_support():
-            return False
-
-        if not self._validate_stream_params(fps, quality, width, height):
-            return False
-
-        # Additional validation for video-specific parameters
-        if video_duration is not None and video_duration <= 0:
-            logging.error("Video duration must be positive")
-            return False
-        if max_frames is not None and max_frames <= 0:
-            logging.error("Max frames must be positive")
-            return False
-        if video_format not in ["mp4", "avi", "webm"]:
-            logging.error("Video format must be one of: mp4, avi, webm")
-            return False
-
-        try:
-            thread = threading.Thread(
-                target=self._stream_video_chunks,
-                args=(
-                    input,
-                    fps,
-                    stream_key,
-                    stream_group_key,
-                    quality,
-                    width,
-                    height,
-                    video_duration,
-                    max_frames,
-                    video_format,
-                    simulate_video_file_stream,
-                ),
-                daemon=True,
-            )
-            self.streaming_threads.append(thread)
-            thread.start()
-            return True
-        except Exception as exc:
-            logging.error("Failed to start video streaming thread: %s", str(exc))
-            return False
-
-    def _stream_video_chunks(
-        self,
-        input: Union[str, int],
-        fps: int = 10,
-        stream_key: Optional[str] = None,
-        stream_group_key: Optional[str] = None,
-        quality: int = 95,
-        width: Optional[int] = None,
-        height: Optional[int] = None,
-        video_duration: Optional[float] = None,
-        max_frames: Optional[int] = None,
-        video_format: str = "mp4",
-        simulate_video_file_stream: bool = False,
-    ) -> None:
-        """Stream video chunks from a video source to Kafka."""
-        quality = max(1, min(100, quality))
-        cap = None
-
-        try:
-            cap, stream_type = self._setup_video_capture(input, width, height)
-
-            # Get video properties including original FPS
-            video_props = self._get_video_properties(cap)
-
-            # Get actual frame dimensions
-            actual_width = int(cap.get(cv2.CAP_PROP_FRAME_WIDTH))
-            actual_height = int(cap.get(cv2.CAP_PROP_FRAME_HEIGHT))
-
-            # Override with specified dimensions if provided
-            if width is not None:
-                actual_width = width
-            if height is not None:
-                actual_height = height
-
-            # Set up video codec
-            fourcc_map = {
-                "mp4": cv2.VideoWriter_fourcc(*"mp4v"),
-                "avi": cv2.VideoWriter_fourcc(*"XVID"),
-                "webm": cv2.VideoWriter_fourcc(*"VP80"),
-            }
-            fourcc = fourcc_map.get(video_format, cv2.VideoWriter_fourcc(*"mp4v"))
-
-            # Calculate chunk limits
-            default_duration = 5.0  # Default chunk duration in seconds
-            if video_duration is not None:
-                chunk_frames = int(fps * video_duration)
-                chunk_duration_seconds = video_duration
-            elif max_frames is not None:
-                chunk_frames = max_frames
-                chunk_duration_seconds = max_frames / fps
-            else:
-                chunk_frames = int(fps * default_duration)
-                chunk_duration_seconds = default_duration
-
-            # Calculate frame skip for RTSP streams
-            frame_skip = 1
-            is_rtsp_stream = isinstance(input, str) and input.startswith("rtsp")
-            if is_rtsp_stream and video_props["original_fps"] > fps:
-                frame_skip = self._calculate_frame_skip(video_props["original_fps"], fps)
-
-            retry_count = 0
-            max_retries = 3
-            chunk_count = 0
-            consecutive_failures = 0
-            max_consecutive_failures = 5
-            global_frame_counter = 0
-
-            while not self._stop_streaming:
-                temp_path = None
-                out = None
-                try:
-                    # Create temporary file for video chunk
-                    with tempfile.NamedTemporaryFile(
-                        suffix=f".{video_format}", delete=False
-                    ) as temp_file:
-                        temp_path = temp_file.name
-
-                    # Create video writer
-                    out = cv2.VideoWriter(
-                        temp_path, fourcc, fps, (actual_width, actual_height)
-                    )
-
-                    if not out.isOpened():
-                        logging.error(f"Failed to open video writer for {temp_path}")
-                        consecutive_failures += 1
-                        if consecutive_failures >= max_consecutive_failures:
-                            logging.error(
-                                "Too many consecutive video writer failures, stopping"
-                            )
-                            break
-                        continue
-
-                    consecutive_failures = 0
-                    frames_in_chunk = 0
-                    chunk_start_frame = global_frame_counter + 1
-                    read_frame_count = 0
-
-                    # Collect frames for this chunk
-                    while frames_in_chunk < chunk_frames and not self._stop_streaming:
-                        frame_start_time = time.time()
-                        ret, frame = cap.read()
-
-                        if not ret:
-                            retry_count += 1
-                            logging.warning(
-                                f"Failed to read frame, retry {retry_count}/{max_retries}"
-                            )
-
-                            try:
-                                cap, retry_count = self._handle_frame_read_failure(
-                                    input, cap, retry_count, max_retries, width, height, simulate_video_file_stream
-                                )
-                            except (RuntimeError, StopIteration):
-                                break
-                            continue
-
-                        retry_count = 0
-                        global_frame_counter += 1
-                        read_frame_count += 1
-
-                        # For RTSP streams, use frame skipping
-                        if is_rtsp_stream:
-                            if read_frame_count % frame_skip != 0:
-                                continue
-
-                        frame = self._resize_frame_if_needed(frame, width, height)
-                        out.write(frame)
-                        frames_in_chunk += 1
-
-                        # For non-RTSP streams, maintain frame rate
-                        if not is_rtsp_stream:
-                            frame_interval = 1.0 / fps
-                            processing_time = time.time() - frame_start_time
-                            sleep_time = max(0, frame_interval - processing_time)
-                            if sleep_time > 0:
-                                time.sleep(sleep_time)
-
-                    # Finalize video chunk
-                    if out is not None:
-                        out.release()
-                        out = None
-
-                    if frames_in_chunk > 0:
-                        # Send video chunk to Kafka
-                        try:
-                            with open(temp_path, "rb") as video_file:
-                                video_bytes = video_file.read()
-
-                            chunk_count += 1
-
-                            # Build metadata using unified method
-                            chunk_metadata = self._build_stream_metadata(
-                                input=input,
-                                stream_key=stream_key,
-                                video_props=video_props,
-                                fps=fps,
-                                quality=quality,
-                                actual_width=actual_width,
-                                actual_height=actual_height,
-                                stream_type=stream_type,
-                                frame_counter=chunk_start_frame,
-                                is_video_chunk=True,
-                                chunk_duration_seconds=chunk_duration_seconds,
-                                chunk_frames=frames_in_chunk,
-                            )
-
-                            success = self.produce_request(
-                                video_bytes,
-                                stream_key,
-                                stream_group_key,
-                                metadata=chunk_metadata,
-                            )
-
-                            if success:
-                                chunk_end_frame = chunk_start_frame + frames_in_chunk - 1
-                                video_timestamp = chunk_metadata["video_timestamp"]
-                                logging.debug(
-                                    f"Successfully sent video chunk {chunk_count} with {frames_in_chunk} frames (frames {chunk_start_frame}-{chunk_end_frame}) at video timestamp {video_timestamp}"
-                                )
-                            else:
-                                logging.warning(
-                                    f"Failed to produce video chunk {chunk_count} to Kafka stream"
-                                )
-
-                        except Exception as e:
-                            logging.error(f"Error reading video chunk file: {str(e)}")
-
-                except Exception as chunk_exc:
-                    logging.error(f"Error processing video chunk: {chunk_exc}")
-                    consecutive_failures += 1
-                    if consecutive_failures >= max_consecutive_failures:
-                        logging.error(
-                            "Too many consecutive chunk processing failures, stopping"
-                        )
-                        break
-                finally:
-                    # Clean up resources
-                    if out is not None:
-                        try:
-                            out.release()
-                        except Exception as e:
-                            logging.warning(f"Error releasing video writer: {e}")
-
-                    if temp_path and os.path.exists(temp_path):
-                        try:
-                            os.unlink(temp_path)
-                        except Exception as e:
-                            logging.warning(
-                                f"Failed to delete temporary file {temp_path}: {str(e)}"
-                            )
-
-                if retry_count >= max_retries:
-                    break
-
-        except Exception as exc:
-            logging.error(f"Error in video streaming thread: {str(exc)}")
-        except KeyboardInterrupt:
-            logging.info("Keyboard interrupt, stopping video streaming")
-        finally:
-            if cap is not None:
-                cap.release()
-                logging.info(f"Released video source: {input}")
-
-            # Clean up stream session
-            if stream_key in self.video_start_times:
-                del self.video_start_times[stream_key]
-
     def stop_streaming(self) -> None:
         """Stop all streaming threads."""
         self._stop_streaming = True
@@ -963,6 +667,44 @@ class ClientStreamUtils:
         self._stop_streaming = False
         logging.info("All streaming threads stopped")
 
+    def get_transmission_stats(self) -> Dict[str, Any]:
+        """Get intelligent transmission statistics.
+
+        Returns:
+            Dictionary with transmission statistics
+        """
+        total_frames = self.frames_sent + self.frames_skipped + self.frames_diff_sent
+        skip_rate = (self.frames_skipped / total_frames) if total_frames > 0 else 0.0
+        diff_rate = (self.frames_diff_sent / total_frames) if total_frames > 0 else 0.0
+        full_rate = (self.frames_sent / total_frames) if total_frames > 0 else 0.0
+
+        return {
+            "frames_sent_full": self.frames_sent,
+            "frames_skipped": self.frames_skipped,
+            "frames_diff_sent": self.frames_diff_sent,
+            "total_frames_processed": total_frames,
+            "skip_rate": skip_rate,
+            "diff_rate": diff_rate,
+            "full_rate": full_rate,
+            "bytes_saved": self.bytes_saved,
+            "intelligent_transmission_enabled": self.enable_intelligent_transmission,
+            "threshold_a": self.threshold_a,
+            "threshold_b": self.threshold_b,
+            "cached_streams": len(self.txh.frame_cache),
+        }
+
+    def reset_transmission_stats(self) -> None:
+        """Reset transmission statistics."""
+        self.frames_sent = 0
+        self.frames_skipped = 0
+        self.frames_diff_sent = 0
+        self.bytes_saved = 0
+        # Clear frame cache to prevent memory leaks
+        if self.txh:
+            self.txh.frame_cache.clear()
+            self.txh.last_frame_hashes.clear()
+        logging.info("Reset transmission statistics and cleared frame cache")
+
     async def close(self) -> None:
         """Close all client connections including Kafka stream."""
         errors = []
@@ -987,4 +729,4 @@ class ClientStreamUtils:
         # Report all errors if any occurred
         if errors:
             error_summary = "\n".join(errors)
-            logging.error("Errors occurred during close: %s", error_summary)
\ No newline at end of file
+            logging.error("Errors occurred during close: %s", error_summary)
